/*!
 * node-bluesky
 * Copyright(c) 2011 Paul O'Fallon <paul@ofallonfamily.com>
 * MIT Licensed
 */
 
var EventEmitter = require('events').EventEmitter;
var util = require('util');

var crypto = require('crypto');
var dateFormat = require('dateformat');
var sax = require("sax");
var skyutil = require("./util");
    
function Queue (q,options) {
  this.options = options;
  this.name = q;
}

util.inherits(Queue,EventEmitter);

Queue.listQueues = function(options, callback) {
  
  var saxStream = sax.createStream(true);
  var insideQueueName = false;
  var queues = [];
  var isError = 0;
  
  var onResponse = function(error, response) {
    if (error) {
      callback(error);
    }
  };
  
  saxStream.on('opentag', function(tag) {
    if (tag.name === "Name") {
      insideQueueName = true;
    } else if (tag.name === "error") { 
      isError++;
    } else if ((tag.name === "code") && (isError === 1)) {
      isError++;
    }
  });
  
  saxStream.on('text', function(s) {
    if (insideQueueName) {
      queues.push(s);
      insideQueueName = false;
    } else if (isError === 2) {
      isError = 0;
      callback(new Error(s));
    }
  });
   
  saxStream.on('closetag', function(name) {
    if (name === "Queues") {
      callback(null,queues);
    }
  });

  saxStream.on('error', function(err) {
    // console.log("ERROR: " + err);
  });
  
  skyutil.doGet({
    'host' : options.account + '.queue.core.windows.net',
    'queryString' : 'comp=list',
    'account' : options.account,
    'key' : options.key,
    'onResponse' : onResponse,
    'responseStream' : saxStream,
    'enhancedAuth' : true,    // Blob & Queue auth differently than Tables
    'authQueryString' : true
  });

};

Queue.createQueue = function(name,options,callback) {

  var saxStream = sax.createStream(true);
  var isError = 0;

  saxStream.on('opentag', function(tag) {
    if (tag.name === 'error') {
      isError++;
    } else if ((tag.name === "code") && (isError === 1)) {
      isError++;
    }
  });

  saxStream.on('text', function(s) {
    if (isError === 2) {
      isError = 0;
      callback(new Error(s));
    }
  });

  saxStream.on('error', function(err) {

  });
  
  var onResponse = function(error, response) {
    if (response.statusCode === 201) {
      var q = new Queue(name,options);
      callback(null,q);
    }
  };
  
  skyutil.doPut({
    'host' : options.account + '.queue.core.windows.net',
    'path' : name,
    'account' : options.account,
    'key' : options.key,
    'onResponse' : onResponse,
    'responseStream' : saxStream,
    'enhancedAuth' : true,
    'metaData' : {
      'Name' : name
    }
  });
  
};

Queue.removeQueue = function(q,options,callback) {

  var saxStream = sax.createStream(true);
  var isError = 0;

  saxStream.on('opentag', function(tag) {
    if (tag.name === 'error') {
      isError++;
    } else if ((tag.name === "code") && (isError === 1)) {
      isError++;
    }
  });

  saxStream.on('text', function(s) {
    if (isError === 2) {
      isError = 0;
      callback(new Error(s));
    }
  });

  saxStream.on('error', function(err) {

  });
  
  var onResponse = function(error, response) {
    if (response.statusCode === 204) {
      callback(null,true);
    }
  };
  
  skyutil.doDelete({
    'host' : options.account + '.queue.core.windows.net',
    'path' : q,
    'account' : options.account,
    'key' : options.key,
    'onResponse' : onResponse,
    'responseStream' : saxStream,
    'enhancedAuth' : true
  });
  
};

Queue.prototype.put = function(m,callback) {

  var saxStream = sax.createStream(true);
  var isError = 0;

  saxStream.on('opentag', function(tag) {
    if (tag.name === 'error') {
      isError++;
    } else if ((tag.name === "code") && (isError === 1)) {
      isError++;
    }
  });

  saxStream.on('text', function(s) {
    if (isError === 2) {
      isError = 0;
      callback(new Error(s));
    }
  });

  saxStream.on('error', function(err) {

  });
  
  var onResponse = function(error, response) {
    if (response.statusCode === 201) {
      callback(null,true);
    }
  };
  
  // TODO:  Support visibilityTimeout and messageTtl
  
  skyutil.doPost({
    'host' : this.options.account + '.queue.core.windows.net',
    'path' : this.name + '/messages',
    'body' : '<QueueMessage><MessageText>' + new Buffer(m).toString('base64') + '</MessageText></QueueMessage>',
    'account' : this.options.account,
    'key' : this.options.key,
    'onResponse' : onResponse,
    'responseStream' : saxStream,
    'enhancedAuth' : true
  });
  
};

Queue.prototype.get = function(callback) {  

  var saxStream = sax.createStream(true);
  var insideMessage = false;
  var currentAttribute = "";
  var message = {};
  var isError = 0;
  
  var onResponse = function(error, response) {
    if (error) {
      callback(error);
    }
  };
  
  saxStream.on('opentag', function(tag) {
    if (tag.name === "QueueMessage") {
      insideMessage = true;
    } else if (insideMessage) {
      currentAttribute = tag.name;
    } else if (tag.name === "error") {
      isError++;
    } else if ((tag.name === "code") && (isError === 1)) {
      isError++;
    }
  });
  
  saxStream.on('text', function(s) {
    if (insideMessage) {
      if (currentAttribute === 'MessageText') {
        message.body = new Buffer(s, 'base64').toString('ascii');
      } else if (currentAttribute === 'MessageId') {
        message.id = s;
      } else {
        // TODO:  Convert timestamps to js date format
        message[currentAttribute.charAt(0).toLowerCase() + currentAttribute.slice(1)] = s;
      } 
    } else if (isError === 2) {
        isError = 0;
        callback(new Error(s));
    }
  });
  
  saxStream.on('closetag', function(name) {
    if (name === "QueueMessage") {
      callback(null,message);
    }
  });

  saxStream.on('error', function(err) {
    // console.log("ERROR: " + err);
  });
  
  skyutil.doGet({
    'host' : this.options.account + '.queue.core.windows.net',
    'path' : this.name + '/messages',
    'account' : this.options.account,
    'key' : this.options.key,
    'onResponse' : onResponse,
    'responseStream' : saxStream,
    'enhancedAuth' : true    // Blob & Queue auth differently than Tables
  });
  
};

Queue.prototype.peek = function(callback) {
  
  var saxStream = sax.createStream(true);
  var insideMessage = false;
  var currentAttribute = "";
  var message = {};
  var isError = 0;
  
  var onResponse = function(error, response) {
    if (error) {
      callback(error);
    }
  };
  
  saxStream.on('opentag', function(tag) {
    if (tag.name === "QueueMessage") {
      insideMessage = true;
    } else if (insideMessage) {
      currentAttribute = tag.name;
    } else if (tag.name === "error") {
      isError++;
    } else if ((tag.name === "code") && (isError === 1) ) {
      isError++;
    }
  });
  
  saxStream.on('text', function(s) {
    if (insideMessage) {
      if (currentAttribute === 'MessageText') {
        message.body = new Buffer(s, 'base64').toString('ascii');
      } else {
        // TODO:  Convert timestamps to js date format
        message[currentAttribute.charAt(0).toLowerCase() + currentAttribute.slice(1)] = s;
      }
    } else if (isError === 2) {
      isError = 0;
      callback(new Error(s));
    }
  });
  
  saxStream.on('closetag', function(name) {
    if (name === "QueueMessage") {
      callback(null,message);
    }
  });

  saxStream.on('error', function(err) {
    // console.log("ERROR: " + err);
  });
  
  skyutil.doGet({
    'host' : this.options.account + '.queue.core.windows.net',
    'path' : this.name + '/messages',
    'queryString' : 'peekonly=true',
    'account' : this.options.account,
    'key' : this.options.key,
    'onResponse' : onResponse,
    'responseStream' : saxStream,
    'enhancedAuth' : true    // Blob & Queue auth differently than Tables
  });
  
};

Queue.prototype.del = function(id,receipt,callback) {

  var saxStream = sax.createStream(true);
  var isError = 0;

  saxStream.on('opentag', function(tag) {
    if (tag.name === 'error') {
      isError++;
    } else if ((tag.name === "code") && (isError === 1)) {
      isError++;
    }
  });

  saxStream.on('text', function(s) {
    if (isError === 2) {
      isError = 0;
      callback(new Error(s));
    }
  });

  saxStream.on('error', function(err) {

  });

  var onResponse = function(error, response) {
    if (response.statusCode === 204) {
      callback(null,true);
    }
  };
  
  // TODO:  Support visibilityTimeout and messageTtl
  
  skyutil.doDelete({
    'host' : this.options.account + '.queue.core.windows.net',
    'path' : this.name + '/messages/' + id,
    'queryString' : 'popreceipt=' + receipt,
    'account' : this.options.account,
    'key' : this.options.key,
    'onResponse' : onResponse,
    'enhancedAuth' : true
  });

};

Queue.prototype.clear = function(callback) {
  
};

Queue.prototype.update = function(m,callback) {
  
};

Queue.prototype.poll = function(t) {

  var that = this;
  
  if (t > 0) {    
    this.interval = setInterval(function() {
      that.get(function(err,msg) {
        if (err) {
          that.emit('error',err);
        } else if (msg) {
          that.emit('message',msg);
        }        
      });
    },t);
  } else {
    clearInterval(this.interval);
  }
  
};

module.exports = Queue;
